from bluer_journal.utils.sync.functions import sync
from bluer_journal.tests.fixtures import journal_checkout


def test_sync(journal_checkout):
    assert sync(
        checklist=True,
        relations=True,
        verbose=True,
    )



from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


BLUER_JOURNAL_REPO = get_env("BLUER_JOURNAL_REPO")

BLUER_JOURNAL_CONFIG = get_env("BLUER_JOURNAL_CONFIG")



from bluer_objects import path

from bluer_journal.classes.page import journal
from bluer_journal.tests.fixtures import journal_checkout


def test_journal(journal_checkout):
    assert journal.path
    assert path.exists(journal.path)

    list_of_pages = journal.list_of_pages()
    assert list_of_pages
    assert "Home" in list_of_pages



NAME = "bluer_journal"

ICON = "📜"

DESCRIPTION = f"{ICON} A journal for the age of AI."

VERSION = "5.81.1"

REPO_NAME = "bluer-journal"

MARQUEE = "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true"

ALIAS = "@journal"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from tqdm import tqdm
from typing import List, Dict, Set
import re

from blueness import module
from bluer_objects import file

from bluer_journal import NAME
from bluer_journal.classes.page import JournalPage
from bluer_journal.classes.journal import journal
from bluer_journal.utils.sync.utils import reformat

from bluer_journal.logger import logger


NAME = module.name(__file__, NAME)


def add_relations(
    dict_of_relations: Dict[str, List[str]],
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.add_relations ...")

    dict_of_relations_inverse: Dict[str, List[str]] = {}
    for related, list_of_relations in tqdm(dict_of_relations.items()):
        for relation in list_of_relations:

            if related in dict_of_relations.get(relation, []):
                continue

            if not file.exists(
                JournalPage(
                    title=relation.replace(" ", "-"),
                    load=False,
                    verbose=verbose,
                ).filename
            ):
                continue

            if relation not in dict_of_relations_inverse:
                dict_of_relations_inverse[relation] = []

            if related in dict_of_relations_inverse[relation]:
                continue

            dict_of_relations_inverse[relation].append(related)

    for relation in dict_of_relations_inverse:
        list_of_similar_relations = [
            relation_
            for relation_ in dict_of_relations_inverse
            if relation_.lower() == relation.lower()
        ]
        if len(list_of_similar_relations) > 1:
            logger.error(
                "similar keywords: {}".format(", ".join(list_of_similar_relations))
            )

            return False

    for relation, list_of_related in tqdm(dict_of_relations_inverse.items()):
        page = JournalPage(
            title=relation.replace(" ", "-"),
            load=True,
            verbose=verbose,
        )

        page.content = [
            f"- [[{related}]]" for related in list_of_related
        ] + page.content

        page.remove_double_blanks()

        if not page.save(generate=False):
            return False

    return True


# pylint: disable=unused-argument
def find_relations(
    page_title: str,
    dict_of_relations: Dict[str, List[str]],
    verbose: bool = False,
) -> bool:
    if page_title in ["Home", "_Sidebar"]:
        return True

    page = JournalPage(
        title=page_title,
        load=True,
        verbose=verbose,
    )
    updated_content: List[str] = []
    for line in page.content:

        if not line.startswith(": "):
            updated_content.append(line)
            continue

        keyword = line.split(": ", 1)[1]
        if not keyword:
            logger.info(f'keyword not found: "{line}"')
            return False

        if bool(re.fullmatch(r"\[\[.+?\]\]", keyword)):
            updated_content.append(line)
            continue

        updated_content.append(f": [[{keyword}]]")

    if updated_content != page.content:
        page.content = updated_content
        if not page.save(generate=False):
            return False

    set_of_relations: Set[str] = set()
    pattern = re.compile(r"\[\[([^\[\]]+)\]\]")
    for line in page.content:
        set_of_relations.update(pattern.findall(line))

    page_title_normalized = page_title.replace("-", " ")
    list_of_relations = [item.replace("-", " ") for item in list(set_of_relations)]
    dict_of_relations[page_title_normalized] = list_of_relations

    if verbose and list_of_relations:
        logger.info(
            "{} -:-> {}".format(
                page_title_normalized,
                ", ".join(list_of_relations),
            )
        )

    return True


def sync_relations(
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.sync_relations ...")

    dict_of_relations: Dict[str, List[str]] = {}
    list_of_pages = journal.list_of_pages(log=verbose)
    for page_title in tqdm(list_of_pages):
        if not find_relations(
            page_title=page_title,
            dict_of_relations=dict_of_relations,
            verbose=verbose,
        ):
            return False

    if not add_relations(
        dict_of_relations=dict_of_relations,
        verbose=verbose,
    ):
        return False

    return reformat(verbose=verbose)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_cd(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@journal",
            "git",
            "cd",
        ],
        "cd git/journal.",
        mono=mono,
    )


def pull_options(mono: bool):
    return xtra("~pull,webhook", mono=mono)


def help_pull(
    tokens: List[str],
    mono: bool,
) -> str:
    options = pull_options(mono=mono)

    return show_usage(
        [
            "@journal",
            "git",
            "pull",
            f"[{options}]",
        ],
        "git -> journal.",
        mono=mono,
    )


def push_options(mono: bool):
    return xtra("dryrun,~push,~sync", mono=mono)


def help_push(
    tokens: List[str],
    mono: bool,
) -> str:

    return show_usage(
        [
            "@journal",
            "git",
            "push",
            f"[{push_options(mono=mono)}]",
        ],
        "journal -> git.",
        mono=mono,
    )


help_functions = {
    "cd": help_cd,
    "pull": help_pull,
    "push": help_push,
}



from bluer_journal.utils.sync.checklist import sync_checklist, find_todo_items
from bluer_journal.tests.fixtures import journal_checkout


def test_sync_checklist_find_todo_items(journal_checkout):
    dict_of_todos = find_todo_items(verbose=True)
    assert isinstance(dict_of_todos, dict)


def test_sync_checklist(journal_checkout):
    assert sync_checklist(verbose=True)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_journal import NAME
from bluer_journal.utils.sync.functions import sync
from bluer_journal.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="sync",
)
parser.add_argument(
    "--checklist",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--relations",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0 |1",
)
args = parser.parse_args()

success = False
if args.task == "sync":
    success = sync(
        checklist=args.checklist == 1,
        relations=args.relations == 1,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import List

from bluer_options.terminal import show_usage, xtra
from bluer_ai.help.generic import help_functions as generic_help_functions

from bluer_journal import ALIAS
from bluer_journal.help.git import help_functions as help_git
from bluer_journal.help.git import pull_options
from bluer_journal.help.open import help_open


def help_sync(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,~push", mono=mono)

    args = [
        "[--checklist 0]",
        "[--relations 0]",
        "[--verbose 1]",
    ]

    return show_usage(
        [
            "@journal",
            "sync",
            f"[{options}]",
            f"[{pull_options(mono=mono)}]",
        ]
        + args,
        "sync journal.",
        mono=mono,
    )


help_functions = generic_help_functions(plugin_name=ALIAS)


help_functions.update(
    {
        "git": help_git,
        "open": help_open,
        "sync": help_sync,
    }
)



from tqdm import tqdm
import re

from blueness import module

from bluer_journal import NAME
from bluer_journal.classes.page import JournalPage
from bluer_journal.classes.journal import journal
from bluer_journal.logger import logger


NAME = module.name(__file__, NAME)


def reformat(
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.reformat ...")

    for title in tqdm(journal.list_of_pages(log=verbose)):
        page = JournalPage(
            title=title,
            load=True,
            verbose=verbose,
        )

        page.content = (
            [
                line
                for line in page.content
                if re.fullmatch(
                    r"- \[\[.+?\]\]",
                    line,
                )
            ]
            + [""]
            + [
                line
                for line in page.content
                if not re.fullmatch(
                    r"- \[\[.+?\]\]",
                    line,
                )
            ]
        )

        page.remove_double_blanks()

        if not page.save(generate=False):
            return False

    return True



from blueness import module

from bluer_journal import NAME
from bluer_journal.utils.sync.checklist import sync_checklist
from bluer_journal.utils.sync.relations import sync_relations
from bluer_journal.logger import logger


NAME = module.name(__file__, NAME)


def sync(
    checklist: bool = True,
    relations: bool = True,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.sync ...")

    if checklist:
        if not sync_checklist(verbose=verbose):
            return False

    if relations:
        if not sync_relations(verbose=verbose):
            return False

    return True
